/*
**	Draw a box
**
**	Compile with:  cc -O -o box box.c
**	Expect to see one warning: "loop not entered at top"
**
**	Note that you need to be root to do these things...
**
**	goran larsson   06-oct-1988
*/

#define LEFT     0x30
#define RIGHT    0x0C
#define LEFTFIX  0x3F
#define RIGHTFIX 0xFC

#include	<dnix/fcodes.h>

/*
** Segment processing of graphic or I/O memory.
** bp = pointer to mapseg structure.
** bc = size of mapseg structure.
** par = parameter describing the action, MAP_IOSH or MAP_GRPH.
*/

#define Segproc(bp, bc, par) dnix(F_SEGP, bp, bc, par, 0)

#define MAP_IOSH 1	/* map in I/O ports and SHared memory */
#define MAP_GRPH 3	/* map in GRaPHic memory              */

/*
** Structure used when mapping in graphic and I/O memory.
*/

struct mapseg {
	char *lad;	/* start of segment mapped in */
	int size;	/* size of segment            */
};

/*
** Graphic memory layout:
**
** The following is true when the screen is in portrait mode.
** Mapseg.lad points to the UPPER LEFT corner. Increasing addresses follows
** line to the right. Each line is 96 bytes (768 pixels) followed by 32 bytes
** used by the font cache. Line two starts at mapseg.lad + 128, and so on.
** The bits in the byte are laid out so that the high bit is the left bit
** on the screen. The single bit in the extreme upper left corner is thus
** set by the operation
**	*mapbuf.lad |= 0x80;
** if mapbuf is declared as struct mapseg and the Segproc call has been used.
**
*/

main ()

{
	int i;
	int j;
	struct mapseg mapbuf;		/* used to map in memory */

	/*
	** Map in the graphic memory, must have uid=0.
	*/

	i = Segproc ( &mapbuf, sizeof(struct mapseg), MAP_GRPH );
	if ( i != sizeof(struct mapseg) ) {
		printf ( "map failed, are you root?\n" );
		exit ( 1 );
	}

	/*
	** Use graphic memory
	*/

	/*
	** wipe
	*/

	line ( mapbuf.lad + 1*128, 96 );
	*(mapbuf.lad + 1*128      ) &= LEFTFIX;
	*(mapbuf.lad + 1*128 + 95 ) &= RIGHTFIX;
	line ( mapbuf.lad + 2*128, 96 );
	*(mapbuf.lad + 2*128      ) &= LEFTFIX;
	*(mapbuf.lad + 2*128 + 95 ) &= RIGHTFIX;
	for ( j = 3; j < 1021; j++ ) {
		*( mapbuf.lad + j*128 ) |= LEFT;
		*( mapbuf.lad + j*128 + 95 ) |= RIGHT;
	}
	line ( mapbuf.lad + 1021*128, 96 );
	*( mapbuf.lad + 1021*128      ) &= LEFTFIX;
	*( mapbuf.lad + 1021*128 + 95 ) &= RIGHTFIX;
	line ( mapbuf.lad + 1022*128, 96 );
	*( mapbuf.lad + 1022*128      ) &= LEFTFIX;
	*( mapbuf.lad + 1022*128 + 95 ) &= RIGHTFIX;
}

/*
** Fill a display line with zero bits, starting at
** address 'base' and filling 'len' bytes.
** This version is very fast.
** Ignore the warning: "loop not entered at top."
** Only real C hackers understands this version of 'line'.
*/

line ( base, len )
register char *base;
register int len;

{
	switch ( len&7 ) {		/* duff's device */
		do {
			*base++ = 0xFF;
	case 7:
			*base++ = 0xFF;
	case 6:
			*base++ = 0xFF;
	case 5:
			*base++ = 0xFF;
	case 4:
			*base++ = 0xFF;
	case 3:
			*base++ = 0xFF;
	case 2:
			*base++ = 0xFF;
	case 1:
			*base++ = 0xFF;
	case 0:;
		} while ( 0<=len-=8 );
	}
}
