/*
**	Show font cache
**
**	Compile with:  cc -O -o cache cache.c
**
**	Note that you need to be root to do these things...
**
**	goran larsson   02-oct-1988
*/

#include	<dnix/fcodes.h>

/*
** Segment processing of graphic or I/O memory.
** bp = pointer to mapseg structure.
** bc = size of mapseg structure.
** par = parameter describing the action, MAP_IOSH or MAP_GRPH.
*/

#define Segproc(bp, bc, par) dnix(F_SEGP, bp, bc, par, 0)

#define MAP_IOSH 1	/* map in I/O ports and SHared memory */
#define MAP_GRPH 3	/* map in GRaPHic memory              */

/*
** Structure used when mapping in graphic and I/O memory.
*/

struct mapseg {
	char *lad;	/* start of segment mapped in */
	int size;	/* size of segment            */
};

/*
** Graphic memory layout:
**
** The following is true when the screen is in portrait mode.
** Mapseg.lad points to the UPPER LEFT corner. Increasing addresses follows
** line to the right. Each line is 96 bytes (768 pixels) followed by 32 bytes
** used by the font cache. Line two starts at mapseg.lad + 128, and so on.
** The bits in the byte are laid out so that the high bit is the left bit
** on the screen. The single bit in the extreme upper left corner is thus
** set by the operation
**	*mapbuf.lad |= 0x80;
** if mapbuf is declared as struct mapseg and the Segproc call has been used.
**
*/

main ()

{
	int i;
	int j;
	struct mapseg mapbuf;		/* used to map in memory */
	register char *base;

	/*
	** Map in the graphic memory, must have uid=0.
	*/

	i = Segproc ( &mapbuf, sizeof(struct mapseg), MAP_GRPH );
	if ( i != sizeof(struct mapseg) ) {
		printf ( "map failed, are you root?\n" );
		exit ( 1 );
	}

	/*
	** copy fontcache
	*/

	base = mapbuf.lad;
	for ( j = 0; j < 1024; j++ ) {
		line ( base ); base += 128;
	}
}

/*
** Move a font cache line into displayed memory.
** Surround it with borders.
*/

line ( base )
char *base;

{
	register char *to = base + 62;
	register char *from = base + 96;
	register int i;

	*(base+60) = 0x00;
	*(base+61) = 0xFF;
	for ( i = 0; i < 32; i++ ) {
		*to++ = *from++;
	}
	*(base+94) = 0xFF;
	*(base+95) = 0x00;
}
