/*
**	Test of graphic memory mapping
**
**	Compile with:  cc -O -o graphmem graphmem.c
**	Expect to see two warnings: "loop not entered at top"
**
**	Note that you need to be root to do these things...
**
**	goran larsson   28-sep-1988
*/

#include	<dnix/fcodes.h>

/*
** Segment processing of graphic or I/O memory.
** bp = pointer to mapseg structure.
** bc = size of mapseg structure.
** par = parameter describing the action, MAP_IOSH or MAP_GRPH.
*/

#define Segproc(bp, bc, par) dnix(F_SEGP, bp, bc, par, 0)

#define MAP_IOSH 1	/* map in I/O ports and SHared memory */
#define MAP_GRPH 3	/* map in GRaPHic memory              */

/*
** Structure used when mapping in graphic and I/O memory.
*/

struct mapseg {
	char *lad;	/* start of segment mapped in */
	int size;	/* size of segment            */
};

/*
** Graphic memory layout:
**
** The following is true when the screen is in portrait mode.
** Mapseg.lad points to the UPPER LEFT corner. Increasing addresses follows
** line to the right. Each line is 96 bytes (768 pixels) followed by 32 bytes
** used by the font cache. Line two starts at mapseg.lad + 128, and so on.
** The bits in the byte are laid out so that the high bit is the left bit
** on the screen. The single bit in the extreme upper left corner is thus
** set by the operation
**	*mapbuf.lad |= 0x80;
** if mapbuf is declared as struct mapseg and the Segproc call has been used.
**
*/

main ()

{
	int i;
	int j;
	struct mapseg mapbuf;		/* used to map in memory */

	/*
	** Map in the graphic memory, must have uid=0.
	*/

	i = Segproc ( &mapbuf, sizeof(struct mapseg), MAP_GRPH );
	if ( i != sizeof(struct mapseg) ) {
		printf ( "map failed, are you root?\n" );
		exit ( 1 );
	}

	/*
	** Use graphic memory
	*/

	/*
	** Write funny pattern at first quarter
	*/

	for ( j = 0; j < 256; j+=8 ) {
		line ( mapbuf.lad + (j+0)*128, 96, 0x80 );
		line ( mapbuf.lad + (j+1)*128, 96, 0xC0 );
		line ( mapbuf.lad + (j+2)*128, 96, 0xE0 );
		line ( mapbuf.lad + (j+3)*128, 96, 0xF0 );
		line ( mapbuf.lad + (j+4)*128, 96, 0xF8 );
		line ( mapbuf.lad + (j+5)*128, 96, 0xFC );
		line ( mapbuf.lad + (j+6)*128, 96, 0xFE );
		line ( mapbuf.lad + (j+7)*128, 96, 0xFF );
	}

	/*
	** Draw 45 degree lines all over the second quarter
	*/

	for ( j = 256; j < 512; j+=8 ) {
		oline ( mapbuf.lad + (j+0)*128, 96, 0x80 );
		oline ( mapbuf.lad + (j+1)*128, 96, 0x40 );
		oline ( mapbuf.lad + (j+2)*128, 96, 0x20 );
		oline ( mapbuf.lad + (j+3)*128, 96, 0x10 );
		oline ( mapbuf.lad + (j+4)*128, 96, 0x08 );
		oline ( mapbuf.lad + (j+5)*128, 96, 0x04 );
		oline ( mapbuf.lad + (j+6)*128, 96, 0x02 );
		oline ( mapbuf.lad + (j+7)*128, 96, 0x01 );
	}

	/*
	** Draw wiggely lines all over the third quarter
	*/

	for ( j = 512; j < 768; j+=8 ) {
		oline ( mapbuf.lad + (j+0)*128, 96, 0x80 );
		oline ( mapbuf.lad + (j+1)*128, 96, 0x20 );
		oline ( mapbuf.lad + (j+2)*128, 96, 0x08 );
		oline ( mapbuf.lad + (j+3)*128, 96, 0x02 );
		oline ( mapbuf.lad + (j+4)*128, 96, 0x02 );
		oline ( mapbuf.lad + (j+5)*128, 96, 0x08 );
		oline ( mapbuf.lad + (j+6)*128, 96, 0x20 );
		oline ( mapbuf.lad + (j+7)*128, 96, 0x80 );
	}

	/*
	** Draw 45 degree crossed fence all over the fourth quarter
	*/

	for ( j = 768; j < 1024; j+=8 ) {
		oline ( mapbuf.lad + (j+0)*128, 96, 0x18 );
		oline ( mapbuf.lad + (j+1)*128, 96, 0x24 );
		oline ( mapbuf.lad + (j+2)*128, 96, 0x42 );
		oline ( mapbuf.lad + (j+3)*128, 96, 0x81 );
		oline ( mapbuf.lad + (j+4)*128, 96, 0x81 );
		oline ( mapbuf.lad + (j+5)*128, 96, 0x42 );
		oline ( mapbuf.lad + (j+6)*128, 96, 0x24 );
		oline ( mapbuf.lad + (j+7)*128, 96, 0x18 );
	}
}

#ifdef THE_OLD_WAY

/*
** Fill a display line with the 8-pixel pattern 'pat', starting at
** address 'base' and filling 'len' bytes.
*/

line ( base, len, pat )
register char *base;
register int len;
register int pat;

{
	while ( len-- ) {
		*base++ = pat;
	}
}

/*
** Or a display line with the 8-pixel mask 'mask', starting at
** address 'base' and or'ing 'len' bytes.
*/

oline ( base, len, mask )
register char *base;
register int len;
register int mask;

{
	while ( len-- ) {
		*base++ |= mask;
	}
}

#else

/*
** Fill a display line with the 8-pixel pattern 'pat', starting at
** address 'base' and filling 'len' bytes.
** This version is very fast.
** Ignore the warning: "loop not entered at top."
** Only real C hackers understands this version of 'line'.
*/

line ( base, len, pat )
register char *base;
register int len;
register int pat;

{
	switch ( len&7 ) {		/* duff's device */
		do {
			*base++ = pat;
	case 7:
			*base++ = pat;
	case 6:
			*base++ = pat;
	case 5:
			*base++ = pat;
	case 4:
			*base++ = pat;
	case 3:
			*base++ = pat;
	case 2:
			*base++ = pat;
	case 1:
			*base++ = pat;
	case 0:;
		} while ( 0<=len-=8 );
	}
}

/*
** Or a display line with the 8-pixel mask 'mask', starting at
** address 'base' and or'ing 'len' bytes.
** This version is very fast.
** Ignore the warning: "loop not entered at top."
** Only real C hackers understands this version of 'oline'.
*/

oline ( base, len, mask )
register char *base;
register int len;
register int mask;

{
	switch ( len&7 ) {		/* duff's device */
		do {
			*base++ |= mask;
	case 7:
			*base++ |= mask;
	case 6:
			*base++ |= mask;
	case 5:
			*base++ |= mask;
	case 4:
			*base++ |= mask;
	case 3:
			*base++ |= mask;
	case 2:
			*base++ |= mask;
	case 1:
			*base++ |= mask;
	case 0:;
		} while ( 0<=len-=8 );
	}
}

#endif THE_OLD_WAY
