/*
**	Clear font cache graphic memory
**
**	Compile with:  cc -O -o wipec wipec.c
**	Expect to see one warning: "loop not entered at top"
**
**	Note that you need to be root to do these things...
**
**	goran larsson   06-oct-1988
*/

#include	<dnix/fcodes.h>

/*
** Segment processing of graphic or I/O memory.
** bp = pointer to mapseg structure.
** bc = size of mapseg structure.
** par = parameter describing the action, MAP_IOSH or MAP_GRPH.
*/

#define Segproc(bp, bc, par) dnix(F_SEGP, bp, bc, par, 0)

#define MAP_IOSH 1	/* map in I/O ports and SHared memory */
#define MAP_GRPH 3	/* map in GRaPHic memory              */

/*
** Structure used when mapping in graphic and I/O memory.
*/

struct mapseg {
	char *lad;	/* start of segment mapped in */
	int size;	/* size of segment            */
};

/*
** Graphic memory layout:
**
** The following is true when the screen is in portrait mode.
** Mapseg.lad points to the UPPER LEFT corner. Increasing addresses follows
** line to the right. Each line is 96 bytes (768 pixels) followed by 32 bytes
** used by the font cache. Line two starts at mapseg.lad + 128, and so on.
** The bits in the byte are laid out so that the high bit is the left bit
** on the screen. The single bit in the extreme upper left corner is thus
** set by the operation
**	*mapbuf.lad |= 0x80;
** if mapbuf is declared as struct mapseg and the Segproc call has been used.
**
*/

main ()

{
	int i;
	int j;
	struct mapseg mapbuf;		/* used to map in memory */

	/*
	** Map in the graphic memory, must have uid=0.
	*/

	i = Segproc ( &mapbuf, sizeof(struct mapseg), MAP_GRPH );
	if ( i != sizeof(struct mapseg) ) {
		printf ( "map failed, are you root?\n" );
		exit ( 1 );
	}

	/*
	** Use graphic memory
	*/

	/*
	** wipe
	*/

	for ( j = 0; j < 1024; j+=8 ) {
		line ( mapbuf.lad + (j+0)*128 + 96, 32 );
		line ( mapbuf.lad + (j+1)*128 + 96, 32 );
		line ( mapbuf.lad + (j+2)*128 + 96, 32 );
		line ( mapbuf.lad + (j+3)*128 + 96, 32 );
		line ( mapbuf.lad + (j+4)*128 + 96, 32 );
		line ( mapbuf.lad + (j+5)*128 + 96, 32 );
		line ( mapbuf.lad + (j+6)*128 + 96, 32 );
		line ( mapbuf.lad + (j+7)*128 + 96, 32 );
	}
}

/*
** Fill a display line with zero bits, starting at
** address 'base' and filling 'len' bytes.
** This version is very fast.
** Ignore the warning: "loop not entered at top."
** Only real C hackers understands this version of 'line'.
*/

line ( base, len )
register char *base;
register int len;

{
	switch ( len&7 ) {		/* duff's device */
		do {
			*base++ = 0;
	case 7:
			*base++ = 0;
	case 6:
			*base++ = 0;
	case 5:
			*base++ = 0;
	case 4:
			*base++ = 0;
	case 3:
			*base++ = 0;
	case 2:
			*base++ = 0;
	case 1:
			*base++ = 0;
	case 0:;
		} while ( 0<=len-=8 );
	}
}
